#!/usr/bin/env python
"""
Dynamic DNS Record Generator (ddrgen), version 1.1.3
Copyright 2005-2006 Michael Farrell

http://micolous.id.au/projects/ddrgen/

This software scans the local network for open SMB/CIFS (Windows File and
Printer Sharing) servers/shares, and maps them to DNS records.  It will
additionally (optionally) write statistics to an XML file so that you can 
display that data on a web page.

Syntax:
 ./ddrgen.py (config file)

  (config file) - The location of the ddrgen.ini file.  See the README file for syntax.

Please note that the forward and reverse zone files will be totally
overwritten on update.

"""

import sys, string, os, time, tempfile
from xml.sax.saxutils import escape
from ConfigParser import ConfigParser

try:
	import ddrghelpers
	from ddrgscanners import *
except:
	print "FATAL: Couldn't import ddrghelpers.  Is it in your Python search path?"
	sys.exit(1)




# check our inputs.
if len(sys.argv) != 2:
	print "FATAL: Oops, you need some more parameters (or less)"
	print "As of ddrgen 1.1.3 a configuration file is used!"
	print __doc__	
	sys.exit(1)
	
conf = ConfigParser()

try:
	conf.read(sys.argv[1])
except Exception, ex:
	print "There was a problem reading the configuration file " + sys.argv[1] + "!"
	raise ex

genStats = ddrghelpers.parseBool(conf.get("ddrgen", "stats"))
genDns = ddrghelpers.parseBool(conf.get("ddrgen", "dns"))
autoNetwork = ddrghelpers.parseBool(conf.get("network", "detect"))

if (not genDns) and (not genStats):
	print "Error: In your current configuration nothing will be done!"
	print "You need to either generate stats or dns, or both."
	sys.exit(1)


autoNameServer = False
djbdns = False

if genDns:
	domain = conf.get("dns", "zone")
	fwdFile = conf.get("dns", "forward")
	zoneFileMode = int(conf.get("dns", "filemode"))
	djbdns = conf.get("dns", "mode").upper() == 'DJBDNS'
	avahi = conf.get("dns", "mode").upper() == 'AVAHI'
	if not djbdns and not avahi:
		revFile = conf.get("dns", "reverse")
	if not avahi:
		updateFreq = conf.get("dns", "updatefreq")
		autoNameServer = conf.get("dns", "ip").upper() == 'AUTO'
		if autoNameServer:
			if not autoNetwork:
				print "ERROR: You must set network.detect to 'yes' to use DNS in 'automatic' mode!"
				sys.exit(1)
		else:
			nsIp = conf.get("dns", "ip")


if genStats:
	genDDRHelper = ddrghelpers.parseBool(conf.get("stats", "ddrhelper"))
	scanForGames = ddrghelpers.parseBool(conf.get("stats", "games"))
	statsFile = conf.get("stats", "file")
	statsFileMode = int(conf.get("stats", "filemode"))

if autoNetwork:
	iface = conf.get("network", "iface")
	iface_info = ddrghelpers.getInterfaceInfo(iface)
	if genDns:
		if autoNameServer:
			nsIp = iface_info[0]
	network = iface_info[1]
	subnet = iface_info[2]
else:
	network = conf.get("network", "network")
	subnet = int(conf.get("network", "subnet"))



ddrghelpers.selfTest(djbdns, scanForGames)

if scanForGames:
	# try to import game modules
	# some installations may be incorrect, or stripped to save space, so we
	# handle this with GRACE.
	try:
		import ut2004, q3a
	except:
		print "WARNING: There was a problem trying to load the game scanning support modules.  This probably means that 'q3a.py' and 'ut2004.py' aren't in your Python search path, or one of their dependancies failed to load correctly."
		print "This program will continue to run, however the 'nogames' option is assumed, so no game servers will be reported."
		scanForGames = False

if (subnet > 31) or (subnet < 8):
	print "FATAL: Invalid subnet, it should be between 8 and 31 (ideally 8, 16 or 24)."
	sys.exit(1)

# check network address
ddrghelpers.checkNetworkAddress(network, subnet)

if genDns and not avahi:
	# Check the NS IP
	ddrghelpers.checkIpAddress(nsIp)

print "Summary of my orders are:"
print " - Scan " + network + "/" + str(subnet) + " for SMB servers"
if genDns:
	if not avahi:
		print " - The expire time of data will be in " + str(updateFreq) + " seconds"
	if not djbdns and not avahi:
		print " - Create a forward DNS in " + fwdFile
		print " - Create a reverse DNS in " + revFile
	elif avahi:
		print " - Create a avahi hosts file in " + fwdFile
	else:
		print " - Create a djbdns-format DNS in " + fwdFile
if genStats:
	print " - Generate stats in " + statsFile
if scanForGames:
	print " - Scan for game servers"
else:
	print " - WILL NOT scan for game servers"
if genDDRHelper:
	print " - DDRHelper support is enabled"
else:
	print " - DDRHelper support is DISABLED"
print "Starting now..."

# make tempfile
tmp = tempfile.mkstemp()
os.close(tmp[0])
tmpfile = tmp[1]
del tmp

print "Scanning for hosts..."
# make a list of up SMB hosts in ddrgen.tmp. add -P0 on nmap to find windows xp firewall'd boxen
os.system("nmap -n -P0 -p 139 \'" + network + "/" + str(subnet) + "\' | grep -B2 \'cp o\' | grep on | awk \'{print $4}\' | grep ':' | sed s/\://g > " + tmpfile)

# ok, now read that file into memory.
fh = open(tmpfile, "r")
ipAddrs = fh.readlines()
fh.close()

print "Searching SMB server shares..."

# resolve IPs to nmb-names (or wins-names, whatever floats your boat)
servers = []
scanners = []
for ip in ipAddrs:
	ip = ip.replace("\n", "")
	current = NetBIOSScannerThread(ip)
	scanners.append(current)
	current.start()

for scanner in scanners:
	scanner.join()
	servers.append(scanner.result)
del scanners


if genDns:
	# try to open the fwd and rev zone files
	try:
		fwdfh = open(fwdFile, "w")
		if not djbdns and not avahi:
			revfh = open(revFile, "w")
	except:
		print "FATAL: Error opening zones files for writing."

	# write headers...

	print "Writing DNS entries..."
	if djbdns:
		# djbdns/tinydns version
		ddrghelpers.iterateDjbDnsRecords(fwdfh, servers, domain, nsIp, time, updateFreq)
	elif avahi:
		# avahi version
		ddrghelpers.iterateAvahiRecords(fwdfh, servers, domain)
	else:
		ddrghelpers.iterateBindRecords(fwdfh, revfh, servers, domain, nsIp, network, subnet, time, updateFreq)
		revfh.close()

	fwdfh.close()
	os.chmod(fwdFile, zoneFileMode)
	if not djbdns and not avahi:
		os.chmod(revFile, zoneFileMode)

# statistics generation...

if genStats:
	if genDDRHelper:
		print "Scanning for DDRHelper hosts..."
		# make a list of up DDRHelper hosts in ddrgen.tmp. add -P0 on nmap to find windows xp firewall'd boxen
		fh = os.popen("nmap -n -P0 -p 6565 \'" + network + "/" + str(subnet) + "\' | grep -B2 \'cp o\' | grep on | awk \'{print $4}\' | grep ':' | sed s/\://g")
		# read the file
		ddrhelpers = fh.readlines()
		fh.close()
	
	if scanForGames:
		print "Generating statistics file, and scanning for shares and game servers.  If firewalls drop rather than reject packets (the 'Steve Gibson' way, like in Windows XP/2003 Firewall, and ZoneAlarm), scanning for games can take a VERY LONG (sometimes infinite) time."
		print "If this takes too long, then you probably have a host on the network like that.  Please kill this python process, and LART the user responsible."
		# Tip to those reading this source code:
		# ping -f works well, particularly if they have popup alerts
		# enabled. :)
		# Of course, you may be booted out of the LAN if you get
		# caught, and some people call it "hacking".
	else:
		print "Generating statistics file, and scanning for shares..."

	try:
		statsfh = open(statsFile + ".new", "w")
	except:
		print "FATAL: Error opening stats file for writing."
		sys.exit(1)

	statsfh.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<?xml-stylesheet type=\"text/xsl\" href=\"stats.xsl\"?>\n<stats>\n")
	statsfh.write("<updated>" + time.strftime("%c") + "</updated>")
	# Generate some statistics.
	scanners = []
	for ip, name in servers:
		isddrhost = genDDRHelper and ip + '\n' in ddrhelpers
		current = ShareScannerThread(ip, name, scanForGames, genDDRHelper, isddrhost)
		scanners.append(current)
		current.start()

	for scanner in scanners:
		scanner.join()
		statsfh.write(scanner.result)
	del scanners

	statsfh.write("</stats>")
	statsfh.close()
	try:
		# delete any old stats file
		os.remove(statsFile)
	except:
		pass
	os.rename(statsFile + ".new", statsFile)
	os.chmod(statsFile, statsFileMode)


# remove our tempfile.
os.remove(tmpfile)

# report that we are done.
print "All done!"
