#!/bin/false
"""ddrghelpers.py: Modules with reusable code in ddrgen.
Copyright 2005-2006 Michael Farrell.
"""

import struct, httplib, os, tempfile
try:
	import iplib
except:
	print "iplib isn't installed!  Please install it, the package is: python-iplib"
	print "Alternatively download it from http://erlug.linux.it/~da/soft/iplib/"
	sys.exit(1)

# define ourself some extra exceptions
class ImproperIpFourException(Exception): pass
class ImproperNetworkAddressException(Exception): pass
class CidrOutOfRangeException(Exception): pass
class OperatingSystemNotSuportedException(Exception): pass

# module functions.
def selfTest(djbdns, scanForGames):
	"""Does a quick check on the system to see if there are any issues with the environment.

	'djbdns' says wether djbdns is used or not.  Similarly with 'scanForGames'.  This determines the level of testing done."""
	
	if djbdns == 1:
		try:
			struct.pack(">Q", long(2**62))
		except:
			print "WARNING: This installation of Python does not support 64-bit 'long long' datatypes.  Please ensure you are running Python 2.2 or later, and that your host system and compiler support this.  ddrgen will attempt to gracefully 'fall back', however errors may occur with the software."
	
	if scanForGames == 1:
		try:
			from twisted.internet.protocol import DatagramProtocol
			from twisted.internet import reactor
		except:
			print "WARNING: Couldn't load twisted networking subsystem.  Please make sure it is installed, and in your Python module path.  You will not be able to scan for game servers without it."
	
	print "Self-test finished.  No fatal errors."
			

def checkIpAddress(ip):
	"""Checks the IP 'ip' to make sure it is correct and well-formed.  Throws ImproperIpFourException on error, returns IP in list if correct."""
	ipArr = ip.split(".")
	
	if len(ipArr) != 4:
		raise ImproperIpFourException, "Network address must have four 'numbers'."
	
	# check the network address and make the bits all integer.
	y = 0
	for x in ipArr:
		x = int(x)
		ipArr[y] = x
		y = y + 1
		if (x < 0) or (x > 255):
			raise ImproperIpFourException, "Octet outside range of 0-255."
	
	return ipArr

def checkNetworkAddress(ip, mask):
	"""Does a basic check to see that a network address is well formed."""
	networkArr = checkIpAddress(ip)
	
	mask = int(mask)
	if (mask > 24) or (mask < 8):
		raise CidrOutOfRangeException, "CIDR 'mask' must be in the range 8 to 24."
	
	if ((networkArr[3] != 0) and (mask <= 24)) or ((networkArr[2] != 0) and (mask <= 16)) or ((networkArr[1] != 0) and (mask <= 8)):
		raise ImproperNetworkAddressException, "Network address isn't correct for the size specified.  Do octets equal zero where needed?"
	
	return networkArr

def isBadNetworkName(name):
	"""Checks if 'name' is "bad".  Particularly needed for some paranoid Primary Domain Controllers, that refuse to state their name in some configurations.
	Returns 1 if bad, 0 otherwise."""
	name = name.lower()
	if (name == "ns") or (name == "ddrlan") or (len(name) <= 1):
		return 1
	else:
		return 0

def makeTai64Timestamp(time):
	"""Makes a TAI-64 timestap from the time object "time".  See http://cr.yp.to/libtai/tai64.html#tai64 for info."""
	# create a decimal tai64 value.  unix time + 2^62.
	taidec = long(time.time()) + long(2**62)
	
	# pack 64-bit long long into string, big endian.
	taihex = struct.pack(">Q", taidec)
	
	
	# convert 8 byte string into 16 bytes base16 string (hex), and return.
	return ''.join(["%02X"%ord(t) for t in taihex])

def iterateDjbDnsRecords(fwdfh, servers, domain, nsIp, time, updateFreq):
	"""Makes some bulk djbdns records.  fwdfh is a file handle for 'data', servers is a list of servers found, domain is the domain of the network."""
	nsfound = 0

	# 'gracefully' fall back if there's no long long (64-bit long) support.
	try:
		timestamp = ":" + makeTai64Timestamp(time)
	except:
		timestamp = ""
	
	fwdfh.write("." + domain + ":" + nsIp + ":ddrlan:" + str(updateFreq) + timestamp + "\n")
	for ip, name in servers:
		if isBadNetworkName(name) == 0:
			fwdfh.write("=" + name + "." + domain + ":" + ip + "\n")
			
			# The nameserver has an SMB server.
			if ip == nsIp:
				nsfound = 1
				

		
	fwdfh.write("+ns." + domain + ":" + nsIp + "\n")
	
def iterateAvahiRecords(fwdfh, servers, domain):
	"""Makes some bulk /etc/avahi/hosts records.  fwdfh is a file handle for 'data', servers is a list of servers found, domain is the domain of the network."""
	for ip, name in servers:
		if len(name) > 0:
			fwdfh.write(ip + "\t" + name + "." + domain + " # ddrgen discovered host\n")

def iterateBindRecords(fwdfh, revfh, servers, domain, nsIp, network, subnet, time, updateFreq):
	"""Makes some bulk BIND-style records.  'fwdfh' and 'revfh' are filehandles, 'time' is a "time" object, and all the rest is self explanitory."""
	networkArr = network.split(".")
	fwdfh.write("$ttl " + str(updateFreq) + """
""" +domain + ".	IN	SOA	ns." + domain + ".	noemail." + domain + """. (
	""" + str(int(time.time())) + """
	""" + str(updateFreq) + """
	""" + str(updateFreq) + """
	""" + str(updateFreq * 5) + """
	5m )

""" + domain + """.	IN	NS	ns.""" + domain + """.
ns.""" + domain + """.	IN	A	""" + nsIp + """
""")

	if subnet < 16:
		arpadomain = str(networkArr[0]) + ".in-addr.arpa"
	else:
		if subnet < 24:
			arpadomain = str(networkArr[1]) + "." + str(networkArr[0]) + ".in-addr.arpa"
		else: # < 24... or waddever.
			arpadomain = str(networkArr[2]) + "." + str(networkArr[1]) + "." + str(networkArr[0]) + ".in-addr.arpa"

	revfh.write("$ttl " + str(updateFreq) + """
""" +arpadomain + ".	IN	SOA	ns." + domain + ".	noemail." + domain + """. (
	""" + str(int(time.time())) + """
	""" + str(updateFreq) + """
	""" + str(updateFreq) + """
	""" + str(updateFreq * 5) + """
	5m )

""" + arpadomain + """.	IN	NS	ns.""" + domain + """.
""")

	# write out servers found
	for ip, name in servers:
		if ddrghelpers.isBadNetworkName(name) == 0:
			fwdfh.write(name + "." + domain + ".	IN	A	" + ip + "\n")
			ipArr = ip.split(".")
			revfh.write(ipArr[3] + "." + ipArr[2] + "." + ipArr[1] + "." + ipArr[0] + ".in-addr.arpa.	IN	PTR	" + name + "." + domain + "." + "\n")

def getServicesFromDDRHelper(ip):
	conn = httplib.HTTPConnection(ip + ":6565")
	conn.request("GET", "/svs")
	result = conn.getresponse()
	buf = result.read()
	conn.close()
	del conn
	
	lines = buf.split('\r\n')
	
	pos = 1
	svs = []
	if lines[0] == 'DDRHelper':
		# file is what we're looking for.
		while pos < len(lines) - 1: # need to ignore a blank line at the eof
			ns = [
				lines[pos], # class type
				lines[pos+1], # service name
				lines[pos+2], # enabled?
				lines[pos+3], # url
				lines[pos+4].replace('<br>', '\r\n'), # description
				lines[pos+5] # port
			]
			
			props = []
			
			# iterate properties
			pos = pos + 6
			while lines[pos] != 'EndService':
				if lines[pos] == 'ServicePropertyString':
					prop = lines[pos+1].split('\t')
					props.append([prop[0], prop[1]])
					del prop
				pos = pos + 2
			
			pos = pos + 1
			
			# add it all up
			ns.append(props)
			svs.append(ns)
			del props, ns
	return svs
	

def parseBool(data):
	if (data.upper() == 'NO') or (data.upper() == 'N') or (data.upper() == 'FALSE') or (data.upper() == 'F') or (str(data) == '0') or (data == False):
		return False
	elif (data.upper() == 'YES') or (data.upper() == 'Y') or (data.upper() == 'TRUE') or (data.upper() == 'T') or (str(data) == '1') or (data == True):
		return True
	else:
		# Failure
		raise ParsingError, str(data) + " is not a valid boolean-like string"

def getInterfaceInfo(iface):
	"""This method implements a way to automatically retrieve information (the IP, network and subnet) of a network interface on the system."""

	iface = iface.replace("`", "").replace("'", "").replace("\"","").replace("\\", "")
	if os.uname()[0].upper() == "LINUX":
		tmp = tempfile.mkstemp()
		os.close(tmp[0])
		os.system("/sbin/ifconfig '" + iface + "' | grep 'inet addr' > " + tmp[1])
		fh = open(tmp[1], "r")
		ifacedata = fh.readline().strip()
		fh.close()
		os.remove(tmp[1])

		ipaddr = ifacedata.split(" ")[1].split(":")[1]
		broadcast = ifacedata.split(" ")[3].split(":")[1]
		netmask = ifacedata.split(" ")[5].split(":")[1]
		subnet = int(iplib.convert_nm(netmask, iplib.NM_BITS))
		
		cidr = iplib.CIDR(ipaddr + "/" + str(subnet))
		network = str(cidr.get_network_ip())
		

		return [ipaddr, network, subnet]
	else:
		raise OperatingSystemNotSupportedException, "Automatic detection of interface information is not supported on this operating system, " + os.uname()[0] + ".  Please use manual configuration instead."


# format is: "Original: (Real Name, Sastisfiable Match)"
def osList():
	return {

	# Windows Vista/Longhorn
	# These versions of Windows report a lot of data in their version strings without help.  TODO: Better handling of Vista-related data.
	"Windows Vista (TM) Ultimate 5600": ("Windows Vista RC1 (Ultimate Edition)", 1),
	
	# Windows Server 2003
	# TODO: Windows 2003 for amd64.  This is only the i386 version.
	"Windows Server 2003 3790": ("Windows 2003 Server (Retail)", 1),
	"Windows Server 2003 3790 Service Pack 1": ("Windows 2003 Server (SP1)", 1),
	
	# Windows XP
	"Windows 5.1": ("Windows XP/i386", 1),
	# This is the "x64" version of Windows XP.  It uses a Win2003 kernel
	# so therefore it is Windows 5.2, and only goes to SP1.
	"Windows XP 3790 Service Pack 1": ("Windows XP Pro/amd64 (SP1)", 1),

	# Windows 2000
	"Windows 5.0": ("Windows 2000", 1),

	# TODO: Windows NT

	# Windows CE
	# I've only done limited testing against Windows CE devices.  The one WinCE device I have easy access to is WinCE 2.0, and doesn't report any version information.
	# When I get more data from hosts running the OS, I can start writing better detection.

	# Windows 9x, ME
	# Having difficulty connecting to Windows 98 test system with smbclient.
	# Operating systems aren't that common anymore anyway, so these aren't a
	# great issue.
	#"Windows 4.9": ("Windows ME", 1), # needs confirmation
	#"Windows 4.1": ("Windows 98", 1),
	#"Windows 4.0": ("Windows 95", 1), # may conflict with NT 4?
	
	"Unix": ("Unix", 0), # Probably samba, unknown OS.  Mac OS X also reports as this.
	"": ""}


def serverToOsList():
	"""This is to try and guess the OS from the patches applied to Samba that are nicely reported to us."""
	return {
	# Debian GNU/Linux.
	"Samba 3.0.23c-Debian": "Debian GNU/Linux (unstable)",
	"Samba 3.0.23b-Debian": "Debian GNU/Linux (unstable)",
	"Samba 3.0.23a-Debian": "Debian GNU/Linux (unstable)",

	# testing/etch (4.0)
	"Samba 3.0.22-Debian": "Debian GNU/Linux (testing/unstable)",

	# stable/sarge (3.1)
	"Samba 3.0.14a-Debian": "Debian GNU/Linux (3.1 \"Sarge\" Release/Current)", # Sarge at release; 20050602; 20060907

	"Samba 3.0.14-Debian": "Debian GNU/Linux", # Sarge
	"Samba 3.0.10-Debian": "Debian GNU/Linux", # Sarge as at 20050430
	
	# Ubuntu Linux
	# Some of these are made up of guesses from packages.ubuntu.com

	# As of Ubuntu Dapper (3.0.22), the Samba daemon reports as a generic one; it does not have the -ubuntu tag set in 'SAMBA_VERSION_VENDOR_SUFFIX'.  This version of Samba is used in the release version of Ubuntu Dapper, and there is no immediate way to discern it from other Samba-running systems with operating systems and distributions other than Ubuntu Linux.

	# Ubuntu Dapper until 20060402
	"Samba 3.0.21c-Ubuntu": "Ubuntu Linux (6.06 \"Dapper\" Pre-Release pre-2006-04-02)",
	# Ubuntu Dapper until 20060324
	"Samba 3.0.21b-Ubuntu": "Ubuntu Linux (6.06 \"Dapper\" Pre-Release pre-2006-03-24)",
	# Ubuntu Dapper until 20060223
	"Samba 3.0.21a-Ubuntu": "Ubuntu Linux (6.06 \"Dapper\" Pre-Release pre-2006-02-23)",
	# Ubuntu Dapper until 20060103 CONFIRMED
	"Samba 3.0.20b-Ubuntu": "Ubuntu Linux (6.06 \"Dapper\" Pre-Release pre-2006-01-03)",
	# Ubuntu Breezy (release), at 20051126 CONFIRMED.
	"Samba 3.0.14a-Ubuntu": "Ubuntu Linux (5.10 \"Breezy\" Release/Current)", 
	## There were no pre-release Samba versions of Ubuntu Breezy ##
	# Ubuntu Hoary as at 20050430 CONFIRMED.
	"Samba 3.0.10-Ubuntu": "Ubuntu Linux (5.04 \"Hoary\" Release/Current)", 
	# Ubuntu Hoary until 20050105
	"Samba 3.0.9-Ubuntu": "Ubuntu Linux (5.04 \"Hoary\" Pre-Release pre-2005-01-05)",
	# Ubuntu Hoary until 20041204
	"Samba 3.0.8-Ubuntu": "Ubuntu Linux (5.04 \"Hoary\" Pre-Release pre-2004-12-04)",
	# Ubuntu Warty as at 20060706, Ubuntu Hoary until 20041118
	"Samba 3.0.7-Ubuntu": "Ubuntu Linux (4.10 \"Warty\" Release/Current; 5.04 \"Hoary\" Pre-Release pre-2004-11-18)",
	# Ubuntu Warty until 20040913
	"Samba 3.0.6-Ubuntu": "Ubuntu Linux (4.10 \"Warty\" Pre-Release pre-2004-09-13)",
	# Ubuntu Warty until 20040826 PRE-RELEASE
	"Samba 3.0.5-Ubuntu": "Ubuntu Linux (4.10 \"Warty\" Pre-Release pre-2004-08-26)",

	
	# Clarkconnect
	"Samba 3.0.10-3.cc": "Clarkconnect Linux",
	
	# Generic ones.  Used by Arch Linux and others
	# Mac OS X doesn't report a specific version, however you should be
	# able to see the outspoken Mac and OSx86 users at a LAN Party. ;)
	# Samba 3.9/4.0 is currently UNSUPPORTED, and can't be scanned by ddrgen properly.
	# Additionally, release candidate versions aren't included here.

	# 2007-05-29: added samba 3.0.23d - 3.0.25a
	"Samba 3.0.25a": "GENERIC",
	"Samba 3.0.25": "GENERIC",
	"Samba 3.0.24": "GENERIC",
	"Samba 3.0.23d": "GENERIC",
	"Samba 3.0.23c": "GENERIC",
	"Samba 3.0.23b": "GENERIC",
	"Samba 3.0.23a": "GENERIC",
	"Samba 3.0.23": "GENERIC",
	"Samba 3.0.22": "GENERIC",
	"Samba 3.0.21d": "GENERIC",
	"Samba 3.0.21c": "GENERIC",
	"Samba 3.0.21b": "GENERIC",
	"Samba 3.0.21a": "GENERIC",
	"Samba 3.0.21": "GENERIC",
	"Samba 3.0.20b": "GENERIC",
	"Samba 3.0.20a": "GENERIC",
	"Samba 3.0.20": "GENERIC",

	"Samba 3.0.14a": "GENERIC",
	"Samba 3.0.14": "GENERIC",
	"Samba 3.0.13": "GENERIC",
	"Samba 3.0.12": "GENERIC",
	"Samba 3.0.11": "GENERIC",
	"Samba 3.0.10": "GENERIC",

	"Samba 3.0.9": "GENERIC",
	"Samba 3.0.8": "GENERIC",
	"Samba 3.0.7": "GENERIC",
	"Samba 3.0.6": "GENERIC",
	"Samba 3.0.5": "GENERIC",
	"Samba 3.0.4": "GENERIC",
	"Samba 3.0.3": "GENERIC",
	"Samba 3.0.2a": "GENERIC",
	"Samba 3.0.2": "GENERIC",
	"Samba 3.0.1": "GENERIC",
	"Samba 3.0.0": "GENERIC",

	"": ""}

